//****************************************************************************
//              Computing the Shannon entropy from recurrence plots
//                   Case of the Logistic map versus mu-parameter
//
//                     C. Letellier, Physical Review Letters, 96, 254102, 2006
//
//                                          Last modification October 26, 2017
//****************************************************************************
#include <iostream>
#include <cmath>
#include <fstream>

using namespace std;

const int Npt = 900;                    // Number of points in your data file

int main() 
{
    int i, j, k, l, m, N;
    int *count;
    double epsi = pow(3.0,0.5)*0.1;
    double Xmin = +1.0e+5;
    double Xmax = -1.0e+5;
    double *x, Dis, Tr, Sh, Prob;
    double R[Npt][Npt];                             // Recurrence plot matrix

    x = new double[Npt];
    count = new int[Npt];
//***************************************************************************
    ifstream data_12 ("datafile.dat");  // Replace with the name of your file

    for (i = 0; i < Npt; i++) 
    {
      data_12 >> x[i];
      Xmin = min(Xmin, x[i]);
      Xmax = max(Xmax, x[i]);
    }
    data_12.close();
    cout << Xmin << " < x < " << Xmax << endl;
    epsi = pow(3.0,0.5)*(Xmax - Xmin)/10.0;                        // Epsilon
    if (epsi < 0.001) 
    {
      Sh = 0.0;
    } 
    else 
    {
      for (i = 0; i < Npt; i++)                 // Initializing of the R_ij
      {
        for (j = 0; j < Npt; j++) 
        {
          R[i][j] = 0.0;
        }
      }
      for (i = 0; i < Npt - 2; i++) 
      {
        for (j = 0; j < Npt - 2; j++) 
        {
          Dis = pow((pow(x[j] - x[i], 2) + pow(x[j+1] - x[i+1],2) 
			                 + pow(x[j+2] - x[i+2], 2)),0.5);
          if (Dis <= epsi) 
	  {
            R[i][j] = 1.0;
          }
        }
      }
      for (i = 0; i < Npt; i++) 
      {
        count[i] = 0;                // Initialisation of the probabilities
      }
      for (i = 1; i < Npt - 1; i++) 
      {
        m = 1;
        for (j = 0; j < Npt - i; j++) 
        {
          if (R[i+j - 1][j] < 0.5) 
	  {
            m++; 
          } 
	  else 
	  {
            count[m]++;
            m = 1;
          }
        }
      }
      N = 0;
      for (i = 0; i < Npt; i++)                   // Total number of events
      {
        N += count[i];
      }
      Sh = 0.0;                            // Computing the Shannon entropy
      for (i = 0; i < Npt; i++) 
      {
        if (count[i] > 0) 
        {
          Prob = double(count[i])/double(N);
          Sh -= Prob*log(Prob);
        }
      }
    } 
    cout << " Shannon entropy = " << Sh << endl;
    return 0;
}
